/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucla.edu, imarkov@cs.ucla.edu
***  Affiliation:   UCLA, Computer Science Department,
***                 Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/




//created on 09/17/98 by Igor Markov

#ifndef _FM_MOVE_MANAGER_BASE_NETCUT2WAYH_  
#define _FM_MOVE_MANAGER_BASE_NETCUT2WAYH_

#include <vector>
  using std::vector;
#include "Partitioners/mmTemplBase.h"
#include "PartLegality/1balanceChk.h"
#include "dcGainCont.h"
#include "PartEvals/netCut2way.h"
#include "Partitioners/svMoveLog.h"

class FMMMbaseNC2w : public  MoveManagerInterface
{
 protected:

    bit_vector                  _lockedModules; 
    bit_vector                  _lockedNets   ; 
    vector<unsigned>            _lockedNetConfigIds;

    unsigned			_numMovable;
    unsigned			_numTerminals;
    unsigned		 	_numParts;	

    NetCut2way                  _eval; 
    DestCentricGainContainer    _gainCo;
    SingleBalanceLegality       _partLeg;
    SVMoveLog                   _moveLog;

    unsigned                    _movingModuleIdx;
    unsigned 			_moveFromPart;
    unsigned			_moveToPart;
    unsigned			_numIllegalMoves;

    bool			_allowCorkingNodes;
    double			_tol;   //used to exclude nodes w/ weight
					//>= the balance tol from the gainCo
					//(they are treated as locked)

    bool			_solnLegalAtStart;	//reset at the
					//begining of each pass, this is 
					//true if the solution was true
					//at the time.

//          virtual bool  pickMove()=0;          
    /* false if a move can not be chosen, true otherwise */
    /* side effect: stores the result as 3 unsigneds */
    /* calls isMoveLegal() and can change gain container, thus not const */

    inline          bool  isMoveLegal() const /* verifies all PartLegalities */
   { return  _partLeg.isLegalMove(_movingModuleIdx,_moveFromPart,_moveToPart);}

//          virtual 
            void  initializeGainBuckets();

            void  computeGainsOfModule(unsigned moduleIdx);
//          virtual void  computeGains()=0;  // using the above

//  inline virtual void   updateGainOfModule
//         (unsigned moduleIdx, unsigned from, unsigned to, int deltaGain)=0;

//         virtual void   updateAffectedModules()=0;
//         virtual void   updateGains()=0


    inline         void   lockModule(unsigned moduleIdx)  
    {   _gainCo.removeAllNodesElements(moduleIdx);
        _lockedModules[moduleIdx]=true;
    }


  public:

    FMMMbaseNC2w(const PartitioningProblem& problem, 
		 bool allowCorkingNodes=false);

    virtual ~FMMMbaseNC2w() {}

    virtual void    setupClip() {  _gainCo.setupClip(); }
    virtual bool     pickMoveApplyIt()=0;  /* calls pickMove(), updateGains(),
			    _moveLog.logMove() and updates *_curPart */

    virtual void     reinitialize();  
    virtual void     resetTo(PartitioningSolution& newSol);
            void     undo(unsigned count) {_moveLog.undo(count); }

    virtual unsigned getCost()       const { return _eval.getTotalCost();}
    virtual double   getCostDouble() const { return getCost();} 

    virtual double   getImbalance() const {return _partLeg.getDiffFromIdeal(); }
 
    virtual double   getViolation() const {return _partLeg.getViolation(); }
    virtual const vector<double>& getAreas() const
                {return _partLeg.getActualBalances();}
};

/*
inline ostream& operator<<(ostream& out, const FMMMbaseNC2w& fmm)
{ fmm.prettyPrint(out); return out;}
*/

#endif
