/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucla.edu, imarkov@cs.ucla.edu
***  Affiliation:   UCLA, Computer Science Department,
***                 Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/



#ifdef _MSC_VER
#pragma warning(disable:4786)
#endif

#include "hgWDims.h"

void HGraphWDimensions::saveAsNodesNetsWts(const char* baseFileName) const
{
    char* fname  = new char[strlen(baseFileName) + 8];
    strcpy(fname, baseFileName);
    char* extPtr = fname + strlen(baseFileName);

    strcpy(extPtr, ".nets");
    ofstream nets(fname);

    strcpy(extPtr, ".nodes");
    ofstream nodes(fname);

    strcpy(extPtr, ".wts");
    ofstream wts (fname);

    strcpy(extPtr, ".aux");
    ofstream aux (fname);


	//write the nodes file
    nodes<<"UCLA nodes 1.0"<<endl;
    nodes<<TimeStamp()<<User()<<Platform()<<endl;

    nodes<<"NumNodes :     "<<setw(10)<<getNumNodes()<<endl;
    nodes<<"NumTerminals : "<<setw(10)<<getNumTerminals()<<endl;

    itHGFNodeGlobal nIt;
    for(nIt = nodesBegin(); nIt != nodesEnd(); nIt++)
    {
	HGFNode& node   = **nIt;
	unsigned nodeId = node.getIndex();
	Symmetry sym    = _nodeSymmetries[nodeId];
	
	nodes<<setw(10)<<node.getName()<<"  ";
	if(getNodeWidth(nodeId) != 0 || getNodeHeight(nodeId) != 0)
	    nodes<<setw(10)<<getNodeWidth(nodeId)<<" "
                 <<setw(10)<<getNodeHeight(nodeId)<<" ";
	if(isTerminal(nodeId))
	    nodes<<"terminal ";
	if(sym.rot90 || sym.y || sym.x)
		nodes << " : "<<sym;
	nodes<<endl;
    }
    nodes.close();


	//write the nets file
    nets<<"UCLA nets 1.0"<<endl;
    nets<<TimeStamp()<<User()<<Platform()<<endl;

    nets<<"NumNets : "<<getNumEdges()<<endl;
    nets<<"NumPins : "<<getNumPins()<<endl;
    
    itHGFEdgeGlobal e;
    for (e=edgesBegin(); e!=edgesEnd(); ++e)
    {
	HGFEdge&	edge   = **e;
	unsigned	edgeId = edge.getIndex();
	
	nets<<"NetDegree : "<<(*e)->getDegree()<<endl;
	unsigned nodeOffset = 0;

        itHGFNodeLocal v;
        for (v=(*e)->srcsBegin(); v!=(*e)->srcsEnd(); v++, nodeOffset++)
        {
	    nets<<setw(10)<<(*v)->getName()<<" O ";

	    Point pOffset = nodesOnEdgePinOffset(nodeOffset, edgeId);
	    if(pOffset.x != 0 || pOffset.y != 0)
		nets<<" : "<<pOffset.x<<" "<<pOffset.y<<endl;
	    else
	 	nets<<endl;
        }
        for (v=(*e)->srcSnksBegin(); v!=(*e)->srcSnksEnd(); ++v, nodeOffset++)
        {
	    nets<<setw(10)<<(*v)->getName()<<" B ";

	    Point pOffset = nodesOnEdgePinOffset(nodeOffset, edgeId);
	    if(pOffset.x != 0 || pOffset.y != 0)
		nets<<" : "<<pOffset.x<<" "<<pOffset.y<<endl;
	    else
	 	nets<<endl;
        }
        for (v=(*e)->snksBegin(); v!=(*e)->snksEnd(); ++v, nodeOffset++)
        {
	    nets<<setw(10)<<(*v)->getName()<<" I ";

	    Point pOffset = nodesOnEdgePinOffset(nodeOffset, edgeId);
	    if(pOffset.x != 0 || pOffset.y != 0)
		nets<<" : "<<pOffset.x<<" "<<pOffset.y<<endl;
	    else
	 	nets<<endl;
        }
    }
    nets.close();

		//write the wts file

    wts<<"UCLA wts 1.0"<<endl;
    wts<<TimeStamp()<<User()<<Platform()<<endl;

    if(getNumWeights() >= 1)
    {
      for(unsigned n=0; n!=getNumNodes(); n++)
      {
	HGFNode& node = *_nodes[n];
	wts<<setw(10)<<node.getName()<<"  ";

        for(unsigned i=0; i!=getNumWeights(); ++i)
	    wts<<" "<<setw(6)<< node.getWeight(i);

        wts<<endl;
      }
    }
    wts.close();

    aux<<"HGraphWPins : "<<baseFileName<<".nets "<<baseFileName<<".nodes "
		<<baseFileName<<".wts"<<endl;
    aux.close();

    delete []fname;
}

