/**************************************************************************
***    
***    Copyright (c) 1995-1998 by ABKGroup, UCLA VLSI CAD Laboratory,
***    UCLA Computer Science Department, Los Angeles, CA 90095-1596 USA, 
***    and by the Regents of the University of California.    
***    All rights reserved.   
***
***    No portion of this package may be used, copied, or transmitted for
***    any purpose, including any research or commercial purpose,
***    without the express written consent of Professor Andrew B. Kahng,
***    UCLA Computer Science Department, 3713 Boelter Hall, Los Angeles,
***    CA 90095-1596 USA.   Any copy made of this file must include this
***    notice.    ABKGroup and the Regents of the University of California
***    retain title to all copies of this file and/or its compiled
***    derivatives.
***
***************************************************************************/

//created by Igor Markov on 05/26/98

#ifndef __1BALANCECHK_H__
#define __1BALANCECHK_H__

#include "partLegXFace.h"

// Note: derived classes should only be used when the precise class
//       is clear (not to compile any methods as virtual)


class SingleBalanceLegality : public PartLegalityXFace
{
protected:

  unsigned       _nParts;
  vector<double> _maxPartBalances;
  vector<double> _minPartBalances;
  vector<double> _targetPartBalances;
  vector<double> _actualPartBalances;

public:

  SingleBalanceLegality
       (const PartitioningProblem& problem,       Partitioning& part);
  SingleBalanceLegality (const PartitioningProblem& problem) ;

   bool isPartLegal() const;  // check current state 
                     // may be slow for single-cell moves in k-way part'ing

   double getViolation()      const { return getMaxViolation(); }
   double getDiffFromIdeal()  const { return getMaxDiffFromIdeal(); }

   double getMaxViolation() const;
   double getMaxDiffFromIdeal() const;

// Reinitializes the checker if the partition changed "non-incrementally"
// Must be redefined in derived classes by calling 
// same method in the immediate parent (with scope resolution operator ::)
  void reinitialize();

  bool isLegalMove(unsigned moduleNumber, unsigned from, unsigned to) const
  {
     double weight=_hg.getNodeByIdx(moduleNumber).getWeight();
     return (_actualPartBalances[from]-weight >= _minPartBalances[from] &&
             _actualPartBalances[to]  +weight <= _maxPartBalances[to] );
  }

  bool isLegalMove(unsigned moduleNumber, PartitionIds old, PartitionIds newIds)
  const
  { return 
      isLegalMove(moduleNumber, old.lowestNumPart(), newIds.lowestNumPart());}

  bool isLegalMoveWithReplication(unsigned moduleNumber,
             PartitionIds oldVal, PartitionIds newVal) const
  { 
    double weight=_hg.getNodeByIdx(moduleNumber).getWeight();
    for(unsigned k=0; k!=_nParts; k++)
    {
       if (oldVal.isInPart(k))
       { 
         if (!newVal.isInPart(k)) 
         if (_actualPartBalances[k]-weight < _minPartBalances[k]) return false;
       }
       else if (newVal.isInPart(k)) 
         if (_actualPartBalances[k]+weight > _maxPartBalances[k]) return false;
    }
    return true;
  }

// These represent incremental update by one move 
// return true if the moved cell does not make things worse
// when inlining, the compiler should detect when the return value is not used

  bool moveModuleTo(unsigned moduleNumber, unsigned from, unsigned to)
  {
    double weight=_hg.getNodeByIdx(moduleNumber).getWeight();
    _actualPartBalances[from]-=weight;
    _actualPartBalances[to]  +=weight;
    return (_actualPartBalances[from] >= _minPartBalances[from] &&
            _actualPartBalances[to]   <= _maxPartBalances[to]   );   
  }
   
  bool
  moveModuleTo(unsigned moduleNumber, PartitionIds old, PartitionIds newIds)
  { return 
    moveModuleTo(moduleNumber, old.lowestNumPart(), newIds.lowestNumPart());}

  bool moveWithReplication(unsigned moduleNumber,
                         PartitionIds oldVal, PartitionIds newVal)
  { 
    double weight=_hg.getNodeByIdx(moduleNumber).getWeight();
    for(unsigned k=0; k!=_nParts; k++)
    {
       if (oldVal.isInPart(k))
          { if (!newVal.isInPart(k)) _actualPartBalances[k]-=weight; }
       else if (newVal.isInPart(k)) _actualPartBalances[k]+=weight;
    }
    return isPartLegal();
  }

// --------- Legality-specific interface

  const vector<double>& getActualBalances() const { return _actualPartBalances;}

  unsigned balance(const Partitioning& fixedConstr); // returns moveCount
  unsigned enforce(const Partitioning& fixedConstr);

  friend ostream& operator<<(ostream&,const SingleBalanceLegality&);
};

ostream& operator<<(ostream&,const SingleBalanceLegality&);

#endif
