/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucla.edu, imarkov@cs.ucla.edu
***  Affiliation:   UCLA, Computer Science Department,
***                 Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/




#include "symOri.h"

OrientationSet::OrientationSet (const Symmetry& sym, const Orient& orient)
/*
   Convert the symmetry, orientation pair into an orientation set.
   
   It would be nice to do something slick here like:
       Start with the single orientation, then generate the
       whole set by applying symmetry generation operations...

   But I don't think there's an easy way to reverse the bits
   in an unsigned, and Symmetry already exposes its
   internal representation, so we'll just look at the bits
   and do a big switch.  Sigh.
*/
{
  unsigned r;
  unsigned tmp1= orient.getAngle();
  unsigned tmp2= (orient.isFaceUp() ? 1 : 0);
  unsigned ori = tmp1 + tmp2;
//unsigned or= orient.getAngle() + (orient.isFaceUp() ? 1 : 0);

//  cout << "sym: " << sym << " orient " << orient << endl;
//  cout << "sym, or: " << unsigned(sym) << " " << or << endl;
  
  switch (unsigned(sym)) {
  case 0:   // No symmetry
    switch (ori) {
    case 1: r = 1; // N
	    break;
    case 0: r = 2; // FN
	    break;
    case 91: r = 4; // E
	    break;
    case 90: r = 8; // FE
	    break;
    case 181: r = 16; // S
	    break;
    case 180: r = 32; // FS
	    break;
    case 271: r = 64; // W
	    break;
    case 270: r = 128; // FW
	    break;
    default:
	cout << "bad value case 0: " << or << endl;    
    }
  case 1:    // X only
    switch (or) {
    case 1: case 180:
      r = 33; // N+FS
      break;
    case 0: case 181:
      r = 18; // FN+S
      break;
    case 90: case 91:
      r = 12; // E+FE
      break;
    case 270: case 271:
      r = 192; // W+FW
      break;
    default:
      cout << "bad value case 1: " << or << endl;
    }
    break;
  case 2:    // Y only
    switch (or) {
    case 0: case 1:
      r = 3; // N+FN
      break;
    case 91: case 270:
      r = 132; // E+FW
      break;
    case 90: case 271:
      r = 72; // FE+W
      break;
    case 180: case 181:
      r = 48; // S+FS
      break;
    default:
      cout << "bad value case 2: " << or << endl;
    }
    break;
  case 3:    // X and Y 
    switch (or) {
    case 0: case 1: case 180: case 181:
      r = 51; // N+FN+S+FS
      break;
    default:
      r = 214; // E+FE+W+FW
    }
    break;
  case 4:    // Rot90 only
    switch (or) {
    case 1: case 91: case 181: case 271:
      r = 85; // N+E+S+W
      break;
    default:
      r = 170; // FN+FE+FS+FW
    }
    break;
    
  default:  // all other cases are the same
    r = 255;
  }
	
  // Now just set all the bits

  N   = r%2; r /= 2;
  FN  = r%2; r /= 2;
  E   = r%2; r /= 2;
  FE  = r%2; r /= 2;
  S   = r%2; r /= 2;
  FS  = r%2; r /= 2;
  W   = r%2; r /= 2;
  FW  = r%2;
  
}
