/**************************************************************************
***    
***    Copyright (c) 1995-1999 by ABKGroup, UCLA VLSI CAD Laboratory,
***    UCLA Computer Science Department, Los Angeles, CA 90095-1596 USA, 
***    and by the Regents of the University of California.    
***    All rights reserved.   
***
***    No portion of this package may be used, copied, or transmitted for
***    any purpose, including any research or commercial purpose,
***    without the express written consent of Professor Andrew B. Kahng,
***    UCLA Computer Science Department, 3713 Boelter Hall, Los Angeles,
***    CA 90095-1596 USA.   Any copy made of this file must include this
***    notice.    ABKGroup and the Regents of the University of California
***    retain title to all copies of this file and/or its compiled
***    derivatives.
***
***************************************************************************/

// Created by Mike Oliver on 3 april 1999
//////////////////////////////////////////////////////////////////////
//
// zTolPart.h: interface for the ZTolPart class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(_ZTOLPART_H_INCLUDED_)
#define _ZTOLPART_H_INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <ABKCommon/abkcommon.h>
#include <Ctainers/bitBoard.h>
#include <Partitioning/partProb.h>
#include <PartEvals/netCut2way.h>

class ZTolPart  
    {
    public:
        
        class Parameters 
            {
            public:
                Verbosity verb;
                unsigned  netIgnoreSize;
                double    nonCutlineIncludeProb;
                unsigned  maxComponentSize;
                unsigned  minPatchToRefine;
                unsigned  numHMetisStarts;
                unsigned  depthLimit;
                unsigned  numBBPartCycles;
                
            Parameters() 
                :verb("silent"),
                 netIgnoreSize(5),
                 nonCutlineIncludeProb(0.2),
                 maxComponentSize(19),
                 minPatchToRefine(4),
                 numHMetisStarts(3),
                 depthLimit(2),
                 numBBPartCycles(100)
                {}
            };
    protected:
        PartitioningProblem       &_problem;
        Parameters                 _params;
        HGraphFixed const         &_hgraph;
        NetCut2way                 _eval;
        BitBoard                   _nodesVisited;
        BitBoard                   _edgesVisited;
        vector<unsigned>           _movables;
        vector<unsigned>           _mapBack;
        vector<unsigned>           _cutline;
        vector<unsigned>           _mapBackCutline;
        vector<vector<unsigned> >  _components;
        RandomRawDouble            _rrd;

        vector<unsigned>           _patch;

        mutable double             _partMax0,_partMax1;
        mutable unsigned           _depthCounter;

        bit_vector                 _isCutline;

        // These are the (possibly negative
        double                     _areaExcess0;
    protected:
        void _findComponents();
        void _findCutline();
        void _initMovables();
        void _expandNode(unsigned nodeIdx);
        void _expandEdge(unsigned edgeIdx);
        void _expandNodeOnePatch(unsigned nodeIdx);
        void _expandEdgeOnePatch(unsigned edgeIdx);
        void _refineComponent(unsigned compIdx);
        void _refinePatch();
        void _findPatchFromEdge(unsigned edgeIdx);
        void _findPatchBFS(unsigned edgeIdx);
        void _finalize();
    public:
        ZTolPart(PartitioningProblem &problem,
                 Parameters params=Parameters());
        virtual ~ZTolPart();
                
    };

#endif // !defined(_ZTOLPART_H_INCLUDED_)

