/**************************************************************************
***    
***    Copyright (c) 1995-1999 by ABKGroup, UCLA VLSI CAD Laboratory,
***    UCLA Computer Science Department, Los Angeles, CA 90095-1596 USA, 
***    and by the Regents of the University of California.    
***    All rights reserved.   
***
***    No portion of this package may be used, copied, or transmitted for
***    any purpose, including any research or commercial purpose,
***    without the express written consent of Professor Andrew B. Kahng,
***    UCLA Computer Science Department, 3713 Boelter Hall, Los Angeles,
***    CA 90095-1596 USA.   Any copy made of this file must include this
***    notice.    ABKGroup and the Regents of the University of California
***    retain title to all copies of this file and/or its compiled
***    derivatives.
***
***************************************************************************/

// Created by Mike Oliver on 28 april 1999

#include "bbPart.h"
#include <hgraphBreakTriangles.h>
#include <discretePrioritizer.h>
void BBPart::_reorderMovables()
    {
    HGraphBreakTriangles noTriGraph(_hgraph);
    vector<unsigned> priorities(_movables.size(),0);
    unsigned k;

    //LOOK! make this a member?
    vector<unsigned> mapBack(noTriGraph.getNumNodes(),UINT_MAX);
    
    for (k=0;k<_movables.size();k++)
        {
        mapBack[_movables[k]]=k;
        }

    // find degrees of nodes wrt subgraph of movables
    // these will be initial priorities
    for (k=0;k<_movables.size();k++)
        {
        HGFNode const &node = noTriGraph.getNodeByIdx(_movables[k]);
        itHGFEdgeLocal iE = node.edgesBegin();
        for (;iE!=node.edgesEnd();iE++)
            {
            bool edgeUsed=false;
            HGFEdge const &edge = *(*iE);
            if (edge.getDegree()!=2)
                continue;
            itHGFNodeLocal iN = edge.nodesBegin();
            for (;iN!=edge.nodesEnd();iN++)
                {
                HGFNode const &otherNode = *(*iN);
                unsigned otherIdx = mapBack[otherNode.getIndex()];
                if (otherIdx != UINT_MAX && otherIdx != k)
                    {
                    edgeUsed=true;
                    break;
                    }
                }
            if (edgeUsed)
                priorities[k] += unsigned(edge.getWeight()+0.5);
            }
        }


    for (k=0;k<_movables.size();k++)
        cout << priorities[k] << " ";
    cout << endl;

    DiscretePrioritizer prior(priorities);

    BitBoard nodeHit(_movables.size());
    vector<unsigned> nbrs;  // immediate neighbors
    vector<unsigned> importantNodes; // will be at head of _movables
    vector<unsigned> sillyNodes;     // will be at end of _movables

    nbrs.reserve(_movables.size());

    unsigned topIdx;

    while ( (topIdx=prior.top()) != UINT_MAX)
        {
        abkfatal(topIdx!=UINT_MAX,"Non-movable index prioritized");

        // highest-priority node removed from prioritizer and
        // added to importantNodes
        importantNodes.push_back(topIdx);
        prior.erase(topIdx);
        //nodeHit.setBit(topIdx);
        HGFNode const &topNode = noTriGraph.getNodeByIdx(_movables[topIdx]);
        mapBack[topNode.getIndex()] = UINT_MAX;

        nbrs.clear();
        itHGFEdgeLocal iE=topNode.edgesBegin();
        for (;iE!=topNode.edgesEnd();iE++)
            {
            HGFEdge const &edge = *(*iE);
            if (edge.getDegree()!=2)
                continue;
            itHGFNodeLocal iN = edge.nodesBegin();
            for (;iN!=edge.nodesEnd();iN++)
                {
                HGFNode const &nbrNode = *(*iN);
                unsigned nbrIdx = mapBack[nbrNode.getIndex()];
                if (nbrIdx != UINT_MAX && nbrIdx != topIdx)
                    {
                    if (!nodeHit.isBitSet(nbrIdx))
                        {
                        nodeHit.setBit(nbrIdx);
                        nbrs.push_back(nbrIdx);
                        }
                    unsigned &thisPrior = priorities[nbrIdx];
                    thisPrior -=
                            unsigned(edge.getWeight()+0.5);

                    abkassert(thisPrior <= 10000,"Priority above "
                        "10000, possible underflow");
                    }
                }
            }

        vector<unsigned>::iterator iNBR = nbrs.begin();
        for (;iNBR!=nbrs.end();iNBR++)
            {
            unsigned nbrIdx = *iNBR;
            prior.requeueToHead(nbrIdx,priorities[nbrIdx]);

            }
        }


    for (k=0;k<importantNodes.size();k++)
        cout << importantNodes[k] << " ";
    cout << endl;
    for (k=0;k<sillyNodes.size();k++)
        cout << sillyNodes[k] << " ";
    cout << endl;
    abkfatal(importantNodes.size()+sillyNodes.size()==_movables.size(),
        "Vertex reordering changed "
        "number of vertices");

    vector<unsigned> temp(_movables);
    _movables.clear();
    _movables.reserve(temp.size());

    for (k=0;k<importantNodes.size();k++)
        _movables.push_back(temp[importantNodes[k]]);

    for (k=0;k<sillyNodes.size();k++)
        _movables.push_back(temp[sillyNodes[k]]);

    _setWeights(); //has to be redone because we've changed the order
                   //of the vertices.  LOOK!  can we eliminate some
                   //redundancy here?

    }


