/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucla.edu, imarkov@cs.ucla.edu
***  Affiliation:   UCLA, Computer Science Department,
***                 Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/





//Small Placement Interface
// - Small Placement Problem
//   created by Andrew Caldwell  981206
// 990429  ilm  added colons to the format 
//              so that it looks similar to SmallPartProblem;
//              added indentation
// 990429  ilm  changed format version number from 1.0 to 1.1
// 990501  ilm  renamed into BaseSmallPlacementProblem

#ifdef _MSC_VER
#pragma warning(disable:4786)
#endif

#include <ctype.h>
#include "HGraphWDims/hgWDims.h"
#include "ABKCommon/pathDelims.h"
#include "baseSmallPlPr.h"

void BaseSmallPlacementProblem:: readAUX(const char* auxFileName)
{
    ifstream auxFile(auxFileName);
    abkfatal(auxFile, "can't open auxfile for reading\n");

    bool success=false;
    char * newPln = NULL;
    char * newPla = NULL;
    char * newPlc = NULL;

    char dir[255];
    const char *auxFileDirEnd=strrchr(auxFileName,pathDelim);
    if (auxFileDirEnd)
    {
       strncpy(dir,auxFileName,auxFileDirEnd-auxFileName+1);
       dir[auxFileDirEnd-auxFileName+1]=0;
    }
    else strcpy(dir,"");    

    bool found=false;
    int  lineNo=1;
    char oneLine[1023];
    char word1[100],word2[100];

    while (!found && !auxFile.eof())
    {
       auxFile >> eathash(lineNo) >> word1
             >> noeol(lineNo) >> word2 >> noeol(lineNo);
       abkfatal(!strcmp(word2,":"),
           " Error in aux file: space-separated column expected\n");

       abkfatal(!strcasecmp(word1,"SmallPlaceProb"),
        " Expecting \"SmallPlaceProb\" -- other formats not recognized\n");
       {
           found=true;       
           auxFile.getline(oneLine,1023);
           unsigned len=strlen(oneLine), fileNum=0;
           bool space=true;
           char *fileNames[3] = {  NULL, NULL, NULL };
           unsigned j;
           for(j=0; j!=len; j++)
           {
              if (isspace(oneLine[j]))
              {
                  if (!space) oneLine[j]='\0';
                  space=true;
              }
              else if (space)
              {
                  space=false;
                  abkfatal(fileNum<3," Too many filenames in AUX file");
                  fileNames[fileNum++]=oneLine+j;
              } 
           }

           for(j=0; j!=fileNum; j++)
           {
              if (strstr(fileNames[j],".pln")) newPln=fileNames[j];   
              if (strstr(fileNames[j],".PLN")) newPln=fileNames[j];   
              if (strstr(fileNames[j],".plc")) newPlc=fileNames[j];   
              if (strstr(fileNames[j],".PLC")) newPlc=fileNames[j];   
              if (strstr(fileNames[j],".pla")) newPla=fileNames[j];   
              if (strstr(fileNames[j],".PLA")) newPla=fileNames[j];   
           }

    	   abkfatal(newPln != NULL && newPlc != NULL,
		"missing a file in smallPartProb");

           char * tmpPln=NULL;
           char * tmpPla=NULL;
           char * tmpPlc=NULL;
      
           unsigned dirLen=strlen(dir);

           if (newPln[0]!=pathDelimWindows && newPln[0] != pathDelimUnix )
           {
               tmpPln=new char[dirLen+strlen(newPln)+1];
               sprintf(tmpPln,"%s%s",dir,newPln);
           }
           else
           {
               tmpPln=new char[strlen(newPln)+1];
               strcat(tmpPln,newPln);
           }       

	   if (newPla != NULL)
           if  ( newPla[0]!=pathDelimWindows && newPla[0] != pathDelimUnix )
           {
               tmpPla=new char[dirLen+strlen(newPla)+1];
               sprintf(tmpPla,"%s%s",dir,newPla);
           }        
           else
           {
               tmpPla=new char[strlen(newPla)+1];
               strcat(tmpPla,newPla);
           }

           if  ( newPlc[0]!=pathDelimWindows && newPlc[0] != pathDelimUnix )
           {
               tmpPlc=new char[dirLen+strlen(newPlc)+1];
               sprintf(tmpPlc,"%s%s",dir,newPlc);
           }        
           else
           {
               tmpPlc=new char[strlen(newPlc)+1];
               strcat(tmpPlc,newPlc);
           }

	   cout<<" Reading PLN File "<<tmpPln<<endl;
           ifstream plnStr(tmpPln);
    	   success =  readPLN(plnStr);
	   plnStr.close();

	   cout<<" Reading PLC File "<<tmpPlc<<endl;
	   ifstream plcStr(tmpPlc);
    	   success &= readPLC(plcStr);
	   plcStr.close();

	   if(tmpPla != NULL)
	   {
	       cout<<" Reading PLA File "<<tmpPla<<endl;
	       ifstream plaStr(tmpPla);
    	       success &=  readPLA(plaStr);
	       plaStr.close();
	   }
	   else
	       _cellWidths = vector<double>(_netlist.getNumCells(),
					    _rows[0].siteInterval);
           if (tmpPln) delete[] tmpPln;
           if (tmpPlc) delete[] tmpPlc;
           if (tmpPla) delete[] tmpPla;
	}
    }

    abkwarn(success,
	"Reading BaseSmallPlacementProblem from files may have failed\n");

}

bool BaseSmallPlacementProblem::readPLN(ifstream& plnStream)
{ 
    abkfatal(plnStream, "can't open .pln file for reading\n");

    int lineNo=1;
    unsigned numNets=0, numCells=0;
    plnStream >> eathash(lineNo) >> needcaseword("UCLA",lineNo)
              >> needcaseword("pln",lineNo) >> skiptoeol;
    lineNo++;
    plnStream >> eathash(lineNo) >> needcaseword("Cells",lineNo)
              >> needcaseword(":",lineNo)
              >> isnumber(lineNo)>> numCells >> needeol(lineNo);
    plnStream >> eathash(lineNo) >> needcaseword("Nets",lineNo)
              >> needcaseword(":",lineNo)
              >> isnumber(lineNo)>> numNets >> needeol(lineNo);

    _netlist.setup(numCells, numNets);
    _solnBuffer.push_back(SmallPlacementSolution(numCells));

    unsigned netIdx;
    unsigned cellIdx;
    Point    newPt;
    unsigned netDegree;

    while(!plnStream.eof() &&  plnStream.peek() != -1)
    {

	netDegree = 0;

        plnStream >> eathash(lineNo) >> needcaseword("Net", lineNo)
	          >> isnumber(lineNo) >> netIdx >> needword(":",lineNo);

	plnStream >> eatblank;

	while(!plnStream.eof() && plnStream.peek() >= '0' &&
		plnStream.peek() <= '9')
	{
	    plnStream >> isnumber(lineNo) >> cellIdx 
		      >> needword("(",lineNo) >> isnumber(lineNo) >> newPt.x 
		      >> isnumber(lineNo) >> newPt.y >> needword(")",lineNo)
		      >> skiptoeol >> eatblank;
	    lineNo++;
	    while (plnStream.peek()==' ' || plnStream.peek()=='\t' ||
	       plnStream.peek()=='\n' || plnStream.peek()=='\r') 
		plnStream.get();
	

	    _netlist.setPinOffset(cellIdx, netIdx, newPt); 
	    netDegree++;
	}

	abkassert(netDegree >= 1, "nets must have degree >= 1");
    }

    return true;
}

bool BaseSmallPlacementProblem::readPLA(ifstream& plaStream)
{ 
    abkfatal(plaStream, "can't open .pla file for reading\n");

    int lineNo=1;
    unsigned numCells=0;

    plaStream >> eathash(lineNo) >> needcaseword("UCLA",lineNo)
              >> needcaseword("pla",lineNo) >> skiptoeol;
    lineNo++;

    plaStream >> eathash(lineNo) >> needcaseword("Cells",lineNo)
              >> needword(":", lineNo)
              >> isnumber(lineNo)>> numCells >> needeol(lineNo);

    abkfatal(numCells == _netlist.getNumCells(),
	"numCells in .pla and .pln do not match\n");

    _cellWidths = vector<double>(numCells, DBL_MAX);

    unsigned cellIdx;
    double   cellWidth;

    for(unsigned c = 0; c < numCells; c++)
    {
        plaStream >> needcaseword("Cell") >> isnumber(lineNo) >> cellIdx
                  >> needword(":",lineNo) 
	          >> isnumber(lineNo) >> cellWidth >> skiptoeol;
       	lineNo++;

	abkassert(cellIdx < numCells, "cellIdx too large");

        _cellWidths[cellIdx] = cellWidth;
    }

    return true;
}

bool BaseSmallPlacementProblem::readPLC(ifstream& plcStream)
{ 
    abkfatal(plcStream, "can't open .plc file for reading");

    int lineNo=1;

    plcStream >> eathash(lineNo) >> needcaseword("UCLA",lineNo)
              >> needcaseword("plc",lineNo) >> skiptoeol;
    lineNo++;

    unsigned numRows;
    unsigned totalNets;
    unsigned netBBoxes; 
    
    plcStream >> eathash(lineNo) >> needcaseword("Rows",lineNo)
              >> needword(":", lineNo)
              >> isnumber(lineNo)>> numRows >> needeol(lineNo);

    plcStream >> eathash(lineNo) >> needcaseword("Total",lineNo)
	      >> needcaseword("Nets") >> needword(":", lineNo)
              >> isnumber(lineNo)
	      >> totalNets >> needeol(lineNo);

    plcStream >> eathash(lineNo) >> needcaseword("Net",lineNo)
	      >> needcaseword("BBoxes") >> needword(":", lineNo)
              >> isnumber(lineNo)
	      >> netBBoxes >> needeol(lineNo);

    plcStream >> eatblank;

    abkfatal(totalNets == _netlist.getNumNets(),
	"num nets from pln and plc files do not match\n");

    _rows = vector<SmallPlacementRow>(numRows); 

    unsigned rowIdx; 
    double   xMin,yMin,xMax,yMax,siteInterval;
    unsigned ctr; 
    for(ctr = 0; ctr < numRows; ctr++)
    {
	plcStream >> eathash(lineNo) >> needcaseword("Row",lineNo)
		  >> isnumber(lineNo) >> rowIdx >> needword(":",lineNo)
		  >> isnumber(lineNo) >> xMin >> isnumber(lineNo) >> yMin
		  >> isnumber(lineNo) >> xMax >> isnumber(lineNo) >> yMax
		  >> isnumber(lineNo) >> siteInterval>>needeol(lineNo);

	abkassert(rowIdx < numRows, "row index too large");

	BBox rowBBox(xMin,yMin,xMax,yMax);
	_rows[rowIdx]=SmallPlacementRow(rowBBox,siteInterval);	
    }

    unsigned netIdx;
    _netTerminalBBoxes = vector<BBox>(totalNets);
    for(ctr = 0; ctr < netBBoxes; ctr++)
    {
	plcStream >> eathash(lineNo) >> needcaseword("Net",lineNo) 
		  >> isnumber(lineNo) >> netIdx >> needword(":",lineNo)
		  >> isnumber(lineNo) >> xMin >> isnumber(lineNo) >> yMin
		  >> isnumber(lineNo) >> xMax >> isnumber(lineNo) >> yMax
		  >> needeol(lineNo);

	abkassert(netIdx < totalNets, "netIdx is too large");
	
	BBox netBBox(xMin,yMin,xMax,yMax);
	_netTerminalBBoxes[netIdx] = netBBox;
    }

    return true;
}

void BaseSmallPlacementProblem::save(const char* baseFileName)
{
    unsigned baseLength = strlen (baseFileName);
    char * newAUX  = new char[baseLength + 5];
    char * newPLN  = new char[baseLength + 5];
    char * newPLC  = new char[baseLength + 5];
    char * newPLA  = new char[baseLength + 5];
    strcpy(newAUX,  baseFileName); strcat(newAUX, ".aux");
    strcpy(newPLN,  baseFileName); strcat(newPLN, ".pln");
    strcpy(newPLC,  baseFileName); strcat(newPLC, ".plc");
    strcpy(newPLA,  baseFileName); strcat(newPLA, ".pla");

    bool success;

    ofstream plnStream(newPLN); 
    abkfatal(plnStream, "can't open pln file for writing");
    cout<<" Writing "<<newPLN<<endl;
    success  = savePLN(plnStream);

    ofstream plcStream(newPLC); 
    abkfatal(plcStream, "can't open plc file for writing");
    cout<<" Writing "<<newPLC<<endl;
    success &= savePLC(plcStream);

    ofstream plaStream(newPLA); 
    abkfatal(plaStream, "can't open pla file for writing");
    cout<<" Writing "<<newPLA<<endl;
    success &= savePLA(plaStream);

    if (success)
    {
       ofstream auxFile(newAUX);
       abkfatal(newAUX," Can't open AUX file for writing");
       cout<<" Writing "<<newAUX<<endl;
       auxFile << "BaseSmallPlaceProb : " << newPLN << " " << newPLC << " "
			<< newPLA <<endl;
    }
 
    delete[] newAUX;
    delete[] newPLN;
    delete[] newPLC;
    delete[] newPLA;
}

bool BaseSmallPlacementProblem::savePLN(ofstream& plnStream)
{
    plnStream << "UCLA pln 1.1 " << endl
              << TimeStamp() << User() << Platform() << endl;

    plnStream << "Cells : "<<_netlist.getNumCells() << endl;
    plnStream << "Nets : "<<_netlist.getNumNets() << endl << endl;

    for(unsigned n = 0; n < _netlist.getNumNets(); n++)
    {
	plnStream << "Net "<< setw(4) << n<<" : ";
	for(unsigned c = 0; c < _netlist.getNumCells(); c++)
        {
	    const Point& offset = _netlist.getPinOffset(c,n);
	    if(offset.x != DBL_MAX)
                 plnStream << setw(4) << c
                           << " ( "   << setw(10) << offset.x
                           << " "     << setw(10) << offset.y <<" )"<<endl; 
	}
	plnStream << endl;
    }
    
    return true;
}

bool BaseSmallPlacementProblem::savePLC(ofstream& plcStream)
{ 
    plcStream<<"UCLA plc 1.1 "<<endl
             <<TimeStamp()<<User()<<Platform()<<endl;

    unsigned n, numBBoxes = 0;
    for(n = 0; n < _netTerminalBBoxes.size(); n++)
	if(!_netTerminalBBoxes[n].isEmpty()) numBBoxes++;

    plcStream<<"Rows : "<<_rows.size()<<endl;
    plcStream<<"Total Nets : "<<_netlist.getNumNets()<<endl;
    plcStream<<"Net BBoxes : "<<numBBoxes<<endl<<endl;

    for(unsigned r = 0; r < _rows.size(); r++)
    {
    	plcStream<<"Row "<<r<<" : "<<_rows[r].xMin<<" "<<_rows[r].yMin
		<<" "<<_rows[r].xMax<<" "<<_rows[r].yMax<<" "
		<<_rows[r].siteInterval<<endl;
    }

    for(n = 0; n < _netTerminalBBoxes.size(); n++)
    {
	const BBox& box = _netTerminalBBoxes[n];
	if(!box.isEmpty())
	    plcStream<<"Net "<<setw(4)<<n
                                  <<" : "<< setw(10) << box.xMin
                                  <<" "<< setw(10) << box.yMin
			          <<" "<< setw(10) << box.xMax
                                  <<" "<< setw(10) << box.yMax << endl;
    }

    return true;
}

bool BaseSmallPlacementProblem::savePLA(ofstream& plaStream)
{ 
    plaStream << "UCLA pla 1.1 " << endl
              << TimeStamp() << User() << Platform() << endl;

    plaStream<<"Cells : "<<_netlist.getNumCells()<<endl<<endl;

    for(unsigned c = 0; c < _cellWidths.size(); c++)
	plaStream<<"Cell "<<setw(4) << c <<" : "
                 << setw(8) << _cellWidths[c]<<endl;
    return true;
}


ostream& operator<<(ostream& os, const BaseSmallPlacementProblem& prob)
{
   os<<"Small Placement Problem"<<endl<<endl;
   os<<"Netlist/Pin Offsets"<<endl;
   os<<prob.getNetlist()<<endl;
   os<<"Cell Widths"<<endl;
   os<<prob.getCellWidths()<<endl;
   os<<"Rows "<<endl;
   for(unsigned r = 0; r < prob._rows.size(); r++)
  	os<<r<<"  "<<prob._rows[r];
   os<<"Net Bounding Boxes"<<endl;
   for(unsigned n = 0; n < prob._netTerminalBBoxes.size(); n++)
	if(!prob._netTerminalBBoxes[n].isEmpty())
	    os<<n<<"  "<<prob._netTerminalBBoxes[n];
	else
	    os<<n<<" none"<<endl;

   return os;
}

