/**************************************************************************
***
***    Copyright (c) 1995-1997 by ABKGroup, UCLA VLSI CAD Laboratory,
***    UCLA Computer Science Department, Los Angeles, CA 90095-1596 USA,
***    and by the Regents of the University of California.
***    All rights reserved.
***
***    No portion of this file may be used, copied, or transmitted for
***    any purpose, including any research or commercial purpose,
***    without the express written consent of Professor Andrew B. Kahng,
***    UCLA Computer Science Department, 3713 Boelter Hall, Los Angeles,
***    CA 90095-1596 USA.   Any copy made of this file must include this
***    notice.    ABKGroup and the Regents of the University of California
***    retain title to all copies of this file and/or its compiled
***    derivatives.
***
***************************************************************************/

// Created: April 10, 1998 by Igor Markov

// CHANGES
// 9804010 ilm 

#ifndef _LOADEDDIE_H_
#define _LOADEDDIE_H_

#include <iostream.h>
#include <vector.h>
#include "abkcommon.h"

class   BiasedRandomSelection;
typedef BiasedRandomSelection  LoadedDie;

// ---------------------------- INTERFACES ------------------------------

class  BiasedRandomSelection
{
   RandomRawUnsigned&   _randu;
   vector<unsigned>    _cumulative;
public:
   BiasedRandomSelection(const vector<double>&   chances, 
                         RandomRawUnsigned&  randu);
   BiasedRandomSelection(const vector<unsigned>& chances,
                         RandomRawUnsigned&  randu);
   operator unsigned() const;
};

// ---------------------------- IMPLEMENTATIONS ------------------------------

BiasedRandomSelection ::BiasedRandomSelection(const vector<unsigned>& chances,
                                              RandomRawUnsigned&  randu)
: _cumulative(chances), _randu(randu)
{
   abkassert(chances.size()>1," No chances ! " );
   vector<unsigned>::iterator it1=_cumulative.begin(),
                            it2=_cumulative.begin()+1;
   for(;it2!=_cumulative.end(); it1++, it2++) (*it2)+=(*it1);
}

BiasedRandomSelection ::BiasedRandomSelection(const vector<double>& chances,
                                              RandomRawUnsigned&  randu)
: _cumulative(chances.size()), _randu(randu)
{
   abkassert(chances.size()>1," No chances ! " );
   vector<double  >::const_iterator it0=chances.begin()+1;
   vector<unsigned>::iterator       it1=_cumulative.begin(),
                                    it2=_cumulative.begin()+1;
   (*it1)=chances.front();
   for(;it2!=_cumulative.end(); it0++, it1++, it2++) (*it2)=(*it1)+(*it0);
}

BiasedRandomSelection::operator unsigned() const
{
   abkfatal(_cumulative.back()>0," Can't apply biased random selection ");
   unsigned drop = _randu % (_cumulative.back());
   return
     upper_bound(_cumulative.begin(), _cumulative.end()-1,drop) 
     - _cumulative.begin();
}


#ifndef AUTHMAR
#define AUTHMAR
static const char
AuthorMarkov[50]="980410, Igor Markov, VLSI CAD ABKGROUP UCLA";
#endif

#endif
