/**************************************************************************
***
***    Copyright (c) 1995-1998 by ABKGroup, UCLA VLSI CAD Laboratory,
***    UCLA Computer Science Department, Los Angeles, CA 90095-1596 USA,
***    and by the Regents of the University of California.
***    All rights reserved.
***
***    No portion of this file may be used, copied, or transmitted for
***    any purpose, including any research or commercial purpose,
***    without the express written consent of Professor Andrew B. Kahng,
***    UCLA Computer Science Department, 3713 Boelter Hall, Los Angeles,
***    CA 90095-1596 USA.   Any copy made of this file must include this
***    notice.    ABKGroup and the Regents of the University of California
***    retain title to all copies of this file and/or its compiled
***    derivatives.
***
***************************************************************************/

#include "constrainingHG.h"

ConstrainingHGraph::ConstrainingHGraph(
   	const vector<CapoBlock*>&       allBlocks,
        const HGraphFixed&              hgraphOfNetlist,
        const Verbosity 		verb)
	: HGraphFixed(hgraphOfNetlist.getNumNodes() + allBlocks.size()*4, 1),
  	  _allBlocks(allBlocks), _netlistHG(hgraphOfNetlist)
{
    static_cast<void>(verb);

    _numTerminals = _netlistHG.getNumTerminals();
//  unsigned numNonTerms  = getNumNodes() - _numTerminals;
    
    _nodeNames = vector<const char*>(getNumNodes());
    _edges.reserve(_netlistHG.getNumEdges() + getNumNodes()*4);

    //all existing nodes are the same...add them in first.

    itHGFNodeGlobal n;
    for(n = _netlistHG.nodesBegin(); n != _netlistHG.nodesEnd(); n++)
    {
	unsigned nId = (*n)->getIndex();

		//note: names are in fact not very essential in this context..
		//though they will be helpful in debugging. This process
		//can be speedup by not having node names in the new HG
	char* newName = new char[strlen((*n)->getName())];
	strcpy(newName, (*n)->getName());
	_namesMap[newName] = nId;
	_nodeNames[nId] = newName;

		//in the interest of speed..this could prob. be done w/
		//a block memory copy..but this would be much more fragile
	_nodes[nId]->setWeight((*n)->getWeight());
    }

    //setup (name and give 0 weight) to the 4*numBlocks new nodes

    unsigned nId;
    for(nId = _netlistHG.getNumNodes(); nId < _nodes.size(); nId++)
    {
	char* newName = new char[20];

	unsigned blockNum = (nId - _netlistHG.getNumNodes())/4;
	unsigned subNum   = (nId - _netlistHG.getNumNodes()) % 4;

	sprintf(newName, "BlockNode%d-%d", blockNum, subNum); 	

	_namesMap[newName] = nId;
	_nodes[nId]->setWeight(0);	
	_nodeNames[nId] = newName;
    }

    //copy all existing edges

    itHGFEdgeGlobal e;
    for(e = _netlistHG.edgesBegin(); e != _netlistHG.edgesEnd(); e++)
    {
	const HGFEdge& origE = **e;
	HGFEdge &newEdge = *(addEdge(origE.getWeight()));

     	itHGFNodeLocal nIt;
	for(nIt = origE.nodesBegin(); nIt != origE.nodesEnd(); nIt++)
	    addSrcSnk(**nIt, newEdge);	
    }

   //add in the new edges

    unsigned numOrigNodes = _netlistHG.getNumNodes();
    for(unsigned blk = 0; blk < _allBlocks.size(); blk++)
    {
	const CapoBlock& block = *_allBlocks[blk];

	const HGFNode& subNode0 = *_nodes[numOrigNodes+blk*4];
	const HGFNode& subNode1 = *_nodes[numOrigNodes+blk*4+1];
	const HGFNode& subNode2 = *_nodes[numOrigNodes+blk*4+2];
	const HGFNode& subNode3 = *_nodes[numOrigNodes+blk*4+3];

	vector<unsigned>::const_iterator cellIds;
	for(cellIds = block.cellIdsBegin(); 
	    cellIds != block.cellIdsEnd(); cellIds++)
	{
	    const HGFNode& node = _netlistHG.getNodeByIdx(*cellIds);
	    unsigned origDegree = node.getDegree();
	    unsigned numToAdd   = 
		static_cast<unsigned>(ceil((float)(origDegree)*0.25));

	    abkfatal3(origDegree == 0 || numToAdd >= 1, origDegree,
		" numToAdd is 0 for a node with degree > 0 ",numToAdd);

	    for(unsigned k = 0; k < numToAdd; k++)
	    {
		HGFEdge *newE = addEdge(1);
		addSrcSnk(subNode0, *newE);
		addSrcSnk(node    , *newE);	
	    }
	    for(k = 0; k < numToAdd; k++)
	    {
		HGFEdge *newE = addEdge(1);
		addSrcSnk(subNode1, *newE);
		addSrcSnk(node    , *newE);	
	    }
	    for(k = 0; k < numToAdd; k++)
	    {
		HGFEdge *newE = addEdge(1);
		addSrcSnk(subNode2, *newE);
		addSrcSnk(node    , *newE);	
	    }
	    for(k = 0; k < numToAdd; k++)
	    {
		HGFEdge *newE = addEdge(1);
		addSrcSnk(subNode3, *newE);
		addSrcSnk(node    , *newE);	
	    }
	}
    }

    finalize();
}

